unit ImageUtil;

interface

uses
    Windows, SysUtils, Classes, Controls, ExtCtrls, Graphics;

type
    TImageSelect = class
    private
        FPaintBox: TPaintBox;
        FDragging: Boolean;
        FSelected: TRect;
        FMouseDownPos: TPoint;
        function GetMiddleLineY(): integer;
    private
        FOrgMouseDown: TMouseEvent;
        FOrgMouseMove: TMouseMoveEvent;
        FOrgMouseUp: TMouseEvent;
        procedure OnMouseDown(Sender: TObject; Button: TMouseButton;
                                Shift: TShiftState; X, Y: Integer);
        procedure OnMouseMove(Sender: TObject; Shift: TShiftState;
                                X, Y: Integer);
        procedure OnMouseUp(Sender: TObject; Button: TMouseButton;
                                Shift: TShiftState; X, Y: Integer);
    public
        constructor Create(PaintBox: TPaintBox);
        destructor Destroy(); override;
        procedure DrawRect(Canvas: TCanvas);
    public
        property Selected: TRect read FSelected;
        property MiddleLineY: integer read GetMiddleLineY;
    end;

implementation

//-----------------------------------------------------------------------------
//  Min
function Min(const A, B: Integer): Integer;
begin
  if A < B then Result := A else Result := B;
end;

//-----------------------------------------------------------------------------
//  Max
function Max(const A, B: Integer): Integer;
begin
  if A >= B then Result := A else Result := B;
end;

//-----------------------------------------------------------------------------
//  RXgN^
constructor TImageSelect.Create(PaintBox: TPaintBox);
begin
    //  yCg{bNXL^
    FPaintBox := PaintBox;
    //  hbOtONA
    FDragging := False;
    //  I͈͏
    FSelected := Rect(-1, -1, -1, -1);
    FMouseDownPos := Point(-1, -1);

    //  ƂƂ̃CxgeL^
    FOrgMouseDown := FPaintBox.OnMouseDown;
    FOrgMouseMove := FPaintBox.OnMouseMove;
    FOrgMouseUp   := FPaintBox.OnMouseUp;

    //  Cxgւ
    FPaintBox.OnMouseDown := OnMouseDown;
    FPaintBox.OnMouseMove := OnMouseMove;
    FPaintBox.OnMouseUp   := OnMouseUp;
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TImageSelect.Destroy();
begin
    //  Cxgɖ߂
    FPaintBox.OnMouseDown := FOrgMouseDown;
    FPaintBox.OnMouseMove := FOrgMouseMove;
    FPaintBox.OnMouseUp   := FOrgMouseUp;

    //  yCg{bNXY
    FPaintBox := nil;

    //  ʃNX̃fXgN^Ăяo
    inherited;
end;

//-----------------------------------------------------------------------------
//  ւpMouseDownCxg
procedure TImageSelect.OnMouseDown(Sender: TObject; Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer);
begin
    //  XMouseDownCxgݒ肳Ă炻Ăяo
    if Assigned(FOrgMouseDown) then
        FOrgMouseDown(Sender, Button, Shift, X, Y)
    ;

    //  ̈ʒuL^
    FMouseDownPos := Point(X, Y);
    FSelected := Rect(FMouseDownPos, FMouseDownPos);
    //  hbOJn
    FDragging := True;
end;

//-----------------------------------------------------------------------------
//  ւpMouseMoveCxg
procedure TImageSelect.OnMouseMove(Sender: TObject; Shift: TShiftState;
                        X, Y: Integer);
var
    R: TRect;
    P1, P2: TPoint;
begin
    //  XMouseMoveCxgݒ肳Ă炻Ăяo
    if Assigned(FOrgMouseMove) then
        FOrgMouseMove(Sender, Shift, X, Y)
    ;

    //  hbOłȂΖ
    if FDragging then
    begin
        //  ŏ̈ʒu
        P1 := FMouseDownPos;
        //  ̈ʒu
        P2 := Point(X, Y);
        //  ĕ`悪KvȔ͈͂߂
        R.Left   := Min(P1.X, P2.X);
        R.Top    := Min(P1.Y, P2.Y);
        R.Right  := Max(P1.X, P2.X);
        R.Bottom := Max(P1.Y, P2.Y);
        //  I͈͂XV
        FSelected := Rect(FMouseDownPos, Point(X, Y));
        //  ĕ`
        FPaintBox.Repaint();
    end;
end;

//-----------------------------------------------------------------------------
//  ւpMouseUpCxg
procedure TImageSelect.OnMouseUp(Sender: TObject; Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer);
var
    R: TRect;
    P1, P2: TPoint;
begin
    //  XMouseUpCxgݒ肳Ă炻Ăяo
    if Assigned(FOrgMouseUp) then
        FOrgMouseUp(Sender, Button, Shift, X, Y)
    ;

    //  hbOłȂΖ
    if FDragging then
    begin
        //  hbO~
        FDragging := False;
        //  ŏ̈ʒu
        P1 := FMouseDownPos;
        //  ̈ʒu
        P2 := Point(X, Y);
        //  ĕ`悪KvȔ͈͂߂
        R.Left   := Min(P1.X, P2.X);
        R.Top    := Min(P1.Y, P2.Y);
        R.Right  := Max(P1.X, P2.X);
        R.Bottom := Max(P1.Y, P2.Y);
        //  I͈͂XV
        FSelected := Rect(FMouseDownPos, Point(X, Y));
        //  ĕ`
        FPaintBox.Repaint();
    end;
end;

//-----------------------------------------------------------------------------
//  I͈͂̐^
function TImageSelect.GetMiddleLineY(): integer;
begin
    with FSelected do
    begin
        Result := Top + (Bottom - Top) div 2;
    end;
end;

//-----------------------------------------------------------------------------
//  I͈͂`
procedure TImageSelect.DrawRect(Canvas: TCanvas);
begin
    //  
    Canvas.Brush.Style := bsClear;

    //  _ŋ``
    Canvas.Pen.Style := psDot;
    Canvas.Rectangle(FSelected);
    Canvas.Pen.Style := psSolid;

    //  ^ɂԂ؂
    Canvas.Pen.Color := clRed;
    with FSelected do
    begin
        Canvas.MoveTo(Left, MiddleLineY);
        Canvas.LineTo(Right, MiddleLineY);
    end;
    Canvas.Pen.Color := clBlack;

    Canvas.Brush.Style := bsSolid;
end;

end.
